import datetime

from betterLogger import colors
from betterLogger import config

standard_tags = {"appname": config.appname, "appauthor": config.appauthor, "appversion": config.appversion,
                 "short_appname": config.short_appname}


# noinspection PyDefaultArgument
def standard_format(string: str, custom_tags: dict[str, any] = {}) -> str:
    datetime_time_now: datetime.datetime = datetime.datetime.now()
    time_now = {"year": datetime_time_now.year, "month": datetime_time_now.month, "day": datetime_time_now.day,
                "hour": datetime_time_now.hour, "minute": datetime_time_now.minute, "second": datetime_time_now.second,
                "microsecond": datetime_time_now.microsecond}
    tags = {}
    tags.update(standard_tags)
    tags.update(custom_tags)
    tags.update(time_now)
    return string.format(**tags)


# noinspection PyDefaultArgument
def colored_format(string: str, allow_color, custom_tags: dict[str, any] = {},
                   custom_colors: dict[str, str] = {}) -> str:
    string = standard_format(string, custom_tags=custom_tags)

    if allow_color:
        cols = {}
        cols.update(colors.str_to_code)
        cols.update(custom_colors)
        for color_name in cols:
            string = string.replace("%" + str(color_name), cols[color_name])

    else:
        cols = {}
        cols.update(colors.str_to_code)
        cols.update(custom_colors)
        for color_name in cols:
            string = string.replace("%" + str(color_name), "")

    return string


__all__ = ["standard_format", "colored_format"]



import logging
# noinspection PyUnresolvedReferences
import sys
from copy import copy
from logging import Manager

from betterLogger.classWithLogger import ClassWithLogger
from betterLogger.consoleHandler import ConsoleHandler
from betterLogger.fileHandler import FileHandler
# Setting up logger ----------------------------------------------------------------------------------------------------
from betterLogger.format_funcs import standard_format
from betterLogger import config
from betterLogger.formatter import Formatter

logging.TRACE = 9
logging.DUMP = 10
logging.DEBUG = 11
# noinspection PyUnresolvedReferences
logging.addLevelName(logging.TRACE, "TRACE")
# noinspection PyUnresolvedReferences
logging.addLevelName(logging.DUMP, "DUMP")


logging.addLevelName(logging.DEBUG, "DEBUG")


root_logger = logging.getLogger("RootLogger")
# noinspection PyUnresolvedReferences
root_logger.setLevel(logging.TRACE)
# noinspection PyUnresolvedReferences,PyProtectedMember
root_logger.trace = lambda message, *args, **kws: \
    root_logger._log(logging.TRACE, message, args, **kws) if root_logger.isEnabledFor(logging.TRACE) else None
# noinspection PyUnresolvedReferences,PyProtectedMember
root_logger.dump = lambda message, *args, **kws: \
    root_logger._log(logging.DUMP, message, args, **kws) if root_logger.isEnabledFor(logging.DUMP) else None

console_handler = ConsoleHandler()
file_handler = FileHandler()
# noinspection PyUnresolvedReferences
console_handler.setLevel(logging.TRACE)
# noinspection PyUnresolvedReferences
file_handler.setLevel(logging.TRACE)


console_formatter = Formatter(use_color=True)
file_formatter = Formatter(use_color=False)
console_handler.setFormatter(console_formatter)
file_handler.setFormatter(file_formatter)


root_logger.addHandler(console_handler)
root_logger.addHandler(file_handler)

# Set our logger to the default ----------------------------------------------------------------------------------------
# noinspection PyTypeChecker
logging.Logger.manager = Manager(root_logger)

# Testing --------------------------------------------------------------------------------------------------------------
if not config.disable_welcome_logging:
    root_logger.critical(standard_format("Welcome to {appname}"))
    root_logger.error(standard_format("Version {appversion}"))
    root_logger.warning(standard_format("Made by {appauthor}"))
    root_logger.info(f"Logger setup and saving to {file_handler.path}")
    # noinspection SpellCheckingInspection
    root_logger.debug("Idk what to put here sooo...")
    root_logger.trace(" 0 /    |  |  +---  |   |   +--+")
    root_logger.dump("/|'     +--+  +--   |   |   |  |")
    root_logger.trace("/ \\     |  |  +---  +-  +-  +--+")
    root_logger.dump("")


root_logger.info(f"Setting log level to {config.log_level}")
root_logger.setLevel(config.log_level)
# ----------------------------------------------------------------------------------------------------------------------


def get_logger(name: str) -> ClassWithLogger:
    root_logger.dump(f"Creating logger with name \"{name}\"")
    return ClassWithLogger(name)


def push_name_to_logger_name_stack(function: callable):
    def wrapper(self, *args, **kwargs):
        self.push_logger_name(function.__name__)
        result = function(self, *args, **kwargs)
        self.pop_logger_name()

        return result
    return wrapper


def reset_logger_name_stack_for_function(function: callable):
    def wrapper(self, *args, **kwargs):
        name_before = copy(self._logger_name_stack)
        self._logger_name_stack.clear()
        result = function(self, *args, **kwargs)
        self._logger_name_stack = name_before

        return result
    return wrapper


def push_name_to_logger_name_stack_custom(name: str):
    def decorator(function: callable):
        def wrapper(self, *args, **kwargs):
            self.push_logger_name(name)
            result = function(self, *args, **kwargs)
            self.pop_logger_name()

            return result
        return wrapper
    return decorator



__all__ = ["get_logger", "ClassWithLogger", "push_name_to_logger_name_stack", "reset_logger_name_stack_for_function",
           "push_name_to_logger_name_stack_custom"]



import copy
from logging import Formatter as _Formatter

from betterLogger import colors
from betterLogger import config
from betterLogger.format_funcs import colored_format


class Formatter(_Formatter):
    def __init__(self, use_color=False):
        _Formatter.__init__(self)
        self.use_color = use_color

    def format(self, record):
        record = copy.deepcopy(record)

        """if record.name in constants.logging.custom_name_per_log_array.keys():
            new_name, new_msg = record.msg.split(": ", maxsplit=1)

            record.name = str(constants.logging.custom_name_per_log_array[record.name]) % new_name
            record.msg = new_msg""" # fix this

        record.msg = colored_format(config.log_format, self.use_color,
                                    custom_tags={"message": record.msg, "logger": self, "level": record.levelname,
                                                 "class_name": record.name},
                                    custom_colors={"LEVEL_COLOR": colors.level_to_code[record.levelname]})
        return _Formatter.format(self, record)


__all__ = ["Formatter"]



reset = "\033[0m"
bold = "\033[1m"

color_changer_thing = "\033[1;%dm"
black, red, green, yellow, blue, magenta, cyan, white = [color_changer_thing % (30 + i) for i in range(8)]

str_to_code = {
    "RESET": reset,
    "BOLD": bold,
    "BLACK": black,
    "RED": red,
    "GREEN": green,
    "YELLOW": yellow,
    "BLUE": blue,
    "MAGENTA": magenta,
    "CYAN": cyan,
    "WHITE": white
}

level_to_code = {
    "CRITICAL": red + bold,
    "ERROR": red,
    "WARNING": yellow,
    "INFO": green,
    "DEBUG": cyan,
    "DUMP": blue,
    "TRACE": black
}


__all__ = ["reset", "bold", "black", "red", "green", "yellow", "blue", "magenta", "cyan", "white", "str_to_code",
           "level_to_code"]



import os
import appdirs


def get_from_environ_else(key, if_not_exists):
    if key in os.environ:
        return os.environ.get(key)
    else:
        return if_not_exists


appname = get_from_environ_else("APPNAME", "BetterLogger")
appauthor = get_from_environ_else("APPAUTHOR", "BetterLogger")
appversion = get_from_environ_else("APPVERSION", "1.0")
short_appname = get_from_environ_else("SHORT_APPNAME", "BL")

log_level = int(get_from_environ_else("LOG_LEVEL", 10))
log_format = get_from_environ_else("LOG_FORMAT", "%LEVEL_COLOR[%BOLD{level: <10}]%RESET %LEVEL_COLOR[%BOLD{class_name: "
                                                 "<32}]%RESET %LEVEL_COLOR {message}%RESET")

save_dir = get_from_environ_else("LOG_SAVE_DIR", appdirs.user_log_dir(appname=appname, appauthor=appauthor,
                                                                      version=appversion))
save_name = get_from_environ_else("LOG_FILE_NAME_FORMAT", "{appname}_{year}-{day}-{hour}-{minute}_{number}.log")

disable_welcome_logging = bool(get_from_environ_else("DISABLE_WELCOME_LOGGING", False))



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name="betterLogger",
    version="1.5",
    description="A great colored logger!",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/GreenJon902/betterLogger",
    author="GreenJon",
    author_email="GreenJon902@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9"
    ],
    packages=["betterLogger"],
    include_package_data=True,
    install_requires=["appdirs"],
    entry_points={
        "console_scripts": [
            "betterLogger=betterLogger.__init__:main",
        ]
    },
)



import logging
from logging import Logger


class ClassWithLogger:
    _created = False
    _logger: Logger = None
    _logger_base_name: str = "GreenJon902IsPog"

    _logger_name_stack: list[str]

    def _create_logger(self):
        self._logger = logging.getLogger(self._logger_base_name)
        self._created = True
        self.log_dump("Created self")

    def _check_logger(self):
        if not self._created:
            self._create_logger()

    def __init__(self, name=None):
        if self._logger_base_name == "GreenJon902IsPog":
            if name is None:
                self._logger_base_name = self.__class__.__name__
            else:
                self._logger_base_name = str(name)


        self._logger_name_stack = list()
        self._check_logger()

    def set_logger_name(self, name: str):
        self._logger_base_name = name
        self._do_logger_name()
        self.log_trace(f"Set base name for self to \"{name}\"")

    def push_logger_name(self, name: str):
        self._logger_name_stack.append(name)
        self._do_logger_name()
        self.log_trace(f"Pushed name \"{name}\" to name stack")

    def pop_logger_name(self) -> str:
        last_name = self._logger_name_stack.pop(-1)
        self._do_logger_name()
        self.log_trace(f"Popped name \"{last_name}\" from name stack")

        return last_name


    def _do_logger_name(self):
        name = self._logger_base_name

        if len(self._logger_name_stack) > 0:
            name += ("." + ".".join(self._logger_name_stack))

        self._logger.name = name

    def log_dump(self, *messages):
        self._check_logger()
        # noinspection PyUnresolvedReferences,PyProtectedMember
        self._logger.log(logging.DUMP, " ".join([str(message) for message in messages]))

    def log_trace(self, *messages):
        self._check_logger()
        # noinspection PyUnresolvedReferences,PyProtectedMember
        self._logger.log(logging.TRACE, " ".join([str(message) for message in messages]))

    def log_debug(self, *messages):
        self._check_logger()
        self._logger.debug(" ".join([str(message) for message in messages]))

    def log_info(self, *messages):
        self._check_logger()
        self._logger.info(" ".join([str(message) for message in messages]))

    def log_warning(self, *messages):
        self._check_logger()
        self._logger.warning(" ".join([str(message) for message in messages]))

    def log_error(self, *messages):
        self._check_logger()
        self._logger.error(" ".join([str(message) for message in messages]))

    def log_critical(self, *messages):
        self._check_logger()
        self._logger.critical(" ".join([str(message) for message in messages]))


__all__ = ["ClassWithLogger"]


if __name__ == '__main__':
    import os
    os.environ["LOG_LEVEL"] = "0"

    # noinspection PyUnresolvedReferences
    import logger

    # noinspection PyRedeclaration
    logger = ClassWithLogger("TestLogger")
    logger.log_info("I should be called Test")

    logger.set_logger_name("Test2")
    logger.log_info("I should be called Test2")

    logger.push_logger_name("Foo")
    logger.log_info("I should be called Test2.Foo")

    logger.push_logger_name("Bar")
    logger.log_info("I should be called Test2.Foo.Bar")

    n = logger.pop_logger_name()
    logger.log_info(f"I should be called Test2.Foo and also \"{n}\" should be \"Bar\"")



import os.path
from logging import FileHandler as _FileHandler

from betterLogger import config
from betterLogger.format_funcs import standard_format


class FileHandler(_FileHandler):
    def __init__(self, encoding=None, delay=False):
        if not os.path.exists(config.save_dir):
            os.makedirs(config.save_dir)

        file_name = standard_format(config.save_name, custom_tags={"number": "{number}"})  # A rather hacky
        #                                                                                    fix
        path = os.path.join(config.save_dir, file_name)

        if "{number}" in file_name:
            n = 0
            while True:
                filename = path.replace("{number}", str(n))
                if not os.path.exists(filename):
                    break
                n += 1
                if n > 1000:  # prevent maybe flooding ?
                    raise Exception("Too many logs, remove them")

            file_name = standard_format(config.save_name, custom_tags={"number": n})
            path = os.path.join(config.save_dir, file_name)

        _FileHandler.__init__(self, path, mode="w", encoding=encoding, delay=delay)
        self.path = path


__all__ = ["FileHandler"]
